__all__ = [
    "BlastingPage",
    "CampfireCookingPage",
    "CraftingPage",
    "EmptyPage",
    "EntityPage",
    "ImagePage",
    "LinkPage",
    "Multiblock",
    "MultiblockPage",
    "Page",
    "PageWithText",
    "PageWithTitle",
    "QuestPage",
    "RelationsPage",
    "SmeltingPage",
    "SmithingPage",
    "SmokingPage",
    "SpotlightPage",
    "StonecuttingPage",
    "TextPage",
]

from .abstract_pages import Page, PageWithText, PageWithTitle
from .pages import (
    BlastingPage,
    CampfireCookingPage,
    CraftingPage,
    EmptyPage,
    EntityPage,
    ImagePage,
    LinkPage,
    Multiblock,
    MultiblockPage,
    QuestPage,
    RelationsPage,
    SmeltingPage,
    SmithingPage,
    SmokingPage,
    SpotlightPage,
    StonecuttingPage,
    TextPage,
)
