{% extends "pages/patchouli/text.html.jinja" %}

{% block inner_body %}
  {{ super() }}
  <h4 class="linkout">
    <a href="{{ page.url }}" target="_blank">{{ page.link_text }}</a>
  </h4>
{% endblock inner_body %}
