{% import "macros/formatting.html.jinja" as fmt with context -%}

<div class="container">
  {# big landing text box - this is optional so addons can disable it if they want #}
  {% if show_landing_text %}
    <header class="jumbotron">
      <h1 class="book-title">{{ book.name }}</h1>
      {{ fmt.styled(book.landing_text) }}
    </header>
  {% endif %}

  {% include "components/table_of_contents.html.jinja" %}

  {# actual book content (ie. all the categories) #}
  <main class="book-body">
    {% for category in book.categories.values() if category.entries.values() +%}
      {% include "category.html.jinja" %}
    {% endfor +%}
  </main>
</div>
