<meta property="og:type" content="website" />
<meta property="og:title" content="{{ _('hexdoc.'~props.modid~'.title') }}" />
{% if icon_href %}<meta property="og:image" content="{{ page_url }}/{{ icon_href }}" />{% endif +%}
<meta property="og:url" content="{{ page_url }}" />
<meta property="og:description" content="{{ _('hexdoc.'~props.modid~'.description') }}" />
<meta property="og:site_name" content="{{ mod_name }} ({{ version }})" />
