"""
A [Jinja](https://jinja.palletsprojects.com/en/3.1.x/)-based documentation generator for [Patchouli](https://github.com/VazkiiMods/Patchouli) books.
"""

from .__version__ import VERSION as VERSION

HEXDOC_MODID = "hexdoc"
