{% extends "redirects/book_link.html.jinja" %}

{% import "macros/formatting.txt.jinja" as fmt_txt with context %}
{% import "macros/textures.html.jinja" as texture_macros with context %}

{% block title -%}
  {{ _('hexdoc.redirect.category.title').format(category.name) }}
{%- endblock title %}

{% block description -%}
  {{ fmt_txt.styled(category.description) }}
{%- endblock description %}

{% block image -%}
  {{ texture_macros.icon_url(category.icon) }}
{%- endblock image %}
