{% import "macros/formatting.jinja" as fmt_base with context %}
{% import "macros/styles.md.jinja" as styles_md with context %}

{# FormatTree handler #}
{% macro styled(value, safe=false) -%}
  {{ fmt_base.styled(
    value,
    safe=safe,
    separator="\n"|safe,
    styles=styles_md,
  ) }}
{%- endmacro %}
