{% import "macros/formatting.html.jinja" as fmt with context %}

<section id="{{ category.id.path }}">
  {% call fmt.maybe_spoilered(category) %}
    {{- fmt.section_header(category, "h2", "category-title") }}
    {{ fmt.styled(category.description) }}
  {% endcall %}

  {% for entry in category.entries.values() if entry.id not in props.entry_id_blacklist +%}
    {% include "entry.html.jinja" %}
  {% endfor +%}
</section>
