{% extends "recipes/base.html.jinja" %}

{% import "macros/recipes.html.jinja" as recipe_macros with context %}
{% import "macros/textures.html.jinja" as texture_macros with context %}

{% block recipe_class -%}
  stonecutting-recipe recipe
{%- endblock %}

{% block title_value -%}
  {{ recipe.result.name }}
{%- endblock %}

{% block content %}
  <div class="stonecutting-input recipe-item-slot">
    {{ recipe_macros.render_ingredients(recipe.ingredient) }}
  </div>

  <div class="stonecutting-result recipe-item-slot">
    {{ texture_macros.render_item(recipe.result, count=recipe.count) }}
  </div>
{% endblock content %}
