{% extends "redirects/book_link.html.jinja" %}

{% import "macros/formatting.txt.jinja" as fmt_txt with context %}
{% import "macros/textures.html.jinja" as texture_macros with context %}

{% block title -%}
  {{ _('hexdoc.redirect.entry.title').format(entry.name) }}
{%- endblock title %}

{% block description -%}
  {%- if entry.first_text_page -%}
    {{ fmt_txt.styled(entry.first_text_page.text) }}
  {%- endif -%}
{%- endblock description %}

{% block image -%}
  {{ texture_macros.icon_url(entry.icon) }}
{%- endblock image %}
